@@ -45,6 +45,7 @@
 #include "util.h"
 #include "private.h"
 #include "time.h"
+#include "time-private.h"
 #include "dhcp-private.h"
 
 struct dhcp_default_transport {
@@ -117,7 +118,7 @@ static bool _dhcp_default_transport_read_handler(struct l_io *io,
 	struct cmsghdr *cmsg;
 	struct iovec iov = { .iov_base = buf, .iov_len = sizeof(buf) };
 	struct msghdr msg = {};
-	unsigned char control[32];
+	unsigned char control[32 + CMSG_SPACE(sizeof(struct timeval))];
 
 	msg.msg_name = &saddr;
 	msg.msg_namelen = sizeof(saddr);
@@ -165,10 +166,12 @@ static bool _dhcp_default_transport_read_handler(struct l_io *io,
 	for (cmsg = CMSG_FIRSTHDR(&msg); cmsg != NULL;
 					cmsg = CMSG_NXTHDR(&msg, cmsg)) {
 		if (cmsg->cmsg_level == SOL_SOCKET &&
-				cmsg->cmsg_type == SCM_TIMESTAMP) {
+				cmsg->cmsg_type == SCM_TIMESTAMP &&
+				cmsg->cmsg_len ==
+				CMSG_LEN(sizeof(struct timeval))) {
 			const struct timeval *tv = (void *) CMSG_DATA(cmsg);
 
-			timestamp = tv->tv_sec * L_USEC_PER_SEC + tv->tv_usec;
+			timestamp = _time_realtime_to_boottime(tv);
 		}
 	}
 
@@ -359,6 +362,9 @@ static int _dhcp_default_transport_bind(struct dhcp_transport *s,
 	if (!transport->io)
 		return -EIO;
 
+	if (transport->udp_fd >= 0)
+		return 0;
+
 	fd = kernel_udp_socket_open(transport->ifname, saddr, transport->port);
 	if (fd < 0)
 		return fd;
@@ -512,6 +518,11 @@ static int _dhcp_default_transport_open(struct dhcp_transport *s, uint32_t xid)
 		return fd;
 
 	transport->io = l_io_new(fd);
+	if (!transport->io) {
+		close(fd);
+		return -EMFILE;
+	}
+
 	l_io_set_close_on_destroy(transport->io, true);
 	l_io_set_read_handler(transport->io,
 					_dhcp_default_transport_read_handler,
@@ -550,6 +561,7 @@ struct dhcp_transport *_dhcp_default_transport_new(uint32_t ifindex,
 	transport->super.ifindex = ifindex;
 	l_strlcpy(transport->ifname, ifname, IFNAMSIZ);
 	transport->port = port;
+	transport->udp_fd = -1;
 
 	return &transport->super;
 }
