@@ -41,13 +41,14 @@
 #include "src/nl80211util.h"
 #include "src/wiphy.h"
 #include "src/frame-xchg.h"
+#include "src/band.h"
 
 #include "linux/nl80211.h"
 
 /* Limit requests per second */
-#define MAX_REQUESTS_PER_SEC		2
+#define MAX_REQUESTS_PER_SEC		2ULL
 /* Microseconds between requests */
-#define MIN_MICROS_BETWEEN_REQUESTS	(1000000 / MAX_REQUESTS_PER_SEC)
+#define MIN_MICROS_BETWEEN_REQUESTS	(1000000ULL / MAX_REQUESTS_PER_SEC)
 
 /* 802.11-2016 Table 9-90 */
 #define REPORT_DETAIL_NO_FIELDS_OR_ELEMS		0
@@ -117,6 +118,7 @@ struct rrm_beacon_req_info {
 /* Per-netdev state */
 struct rrm_state {
 	struct station *station;
+	uint32_t watch_id;
 	uint32_t ifindex;
 	uint64_t wdev_id;
 	struct rrm_request_info *pending;
@@ -277,7 +279,7 @@ static size_t build_report_for_bss(struct rrm_beacon_req_info *beacon,
 	struct rrm_beacon_report *report = (struct rrm_beacon_report *) to;
 
 	report->oper_class = beacon->oper_class;
-	report->channel = scan_freq_to_channel(bss->frequency, NULL);
+	report->channel = band_freq_to_channel(bss->frequency, NULL);
 	report->scan_start_time = L_CPU_TO_LE64(beacon->scan_start_time);
 	report->duration = L_CPU_TO_LE16(beacon->duration);
 	report->frame_info = rrm_phy_type(bss);
@@ -309,7 +311,7 @@ static size_t build_report_for_bss(struct rrm_beacon_req_info *beacon,
 static bool bss_in_request_range(struct rrm_beacon_req_info *beacon,
 					struct scan_bss *bss)
 {
-	uint8_t channel = scan_freq_to_channel(bss->frequency, NULL);
+	uint8_t channel = band_freq_to_channel(bss->frequency, NULL);
 
 	/* Must be a table measurement */
 	if (beacon->channel == 0 || beacon->channel == 255)
@@ -435,10 +437,10 @@ static void rrm_handle_beacon_scan(struct rrm_state *rrm,
 		.duration = beacon->duration,
 		.duration_mandatory = test_bit(&beacon->info.mode, 4),
 	};
-	enum scan_band band = scan_oper_class_to_band(NULL, beacon->oper_class);
+	enum band_freq band = band_oper_class_to_band(NULL, beacon->oper_class);
 	uint32_t freq;
 
-	freq = scan_channel_to_freq(beacon->channel, band);
+	freq = band_channel_to_freq(beacon->channel, band);
 	scan_freq_set_add(freqs, freq);
 
 	if (passive)
@@ -484,7 +486,7 @@ static bool rrm_verify_beacon_request(const uint8_t *request, size_t len)
 	}
 
 	/* Check this is a valid operating class */
-	if (!scan_oper_class_to_band(NULL, request[0]))
+	if (!band_oper_class_to_band(NULL, request[0]))
 		return false;
 
 	return true;
@@ -569,7 +571,7 @@ static void rrm_handle_beacon_request(struct rrm_state *rrm,
 
 			break;
 		case RRM_BEACON_REQ_SUBELEM_ID_REPORTING_DETAIL:
-			if (length != 1) {
+			if (length < 1) {
 				l_error("Invalid length in reporting detail");
 				goto reject_refused;
 			}
@@ -577,6 +579,11 @@ static void rrm_handle_beacon_request(struct rrm_state *rrm,
 			detail = l_get_u8(data);
 			break;
 		case RRM_BEACON_REQ_SUBELEM_ID_BEACON_REPORTING:
+			if (length < 2) {
+				l_error("Invalid length in Beacon Reporting");
+				goto reject_refused;
+			}
+
 			/*
 			 * 802.11-2016 9.4.2.21.7
 			 *
@@ -584,9 +591,19 @@ static void rrm_handle_beacon_request(struct rrm_state *rrm,
 			 *  present in a Beacon request for repeated
 			 *  measurements; otherwise it is not present"
 			 *
-			 * Since repeated measurements are not supported we can
-			 * reject this request now.
+			 * However, some implementations send an all-zero
+			 * Beacon Reporting subelement.  Reporting Condition
+			 * of zero is 'default, used when the Beacon Reporting
+			 * subelement is not included in a Beacon request).'
+			 * Treat such elements as if they're not present.
+			 *
+			 * Otherwise, since repeated measurements are not
+			 * supported we can reject this request now.
 			 */
+			if (l_get_u8(data) != 0)
+				goto reject_incapable;
+
+			break;
 		case RRM_BEACON_REQ_SUBELEM_ID_AP_CHAN_REPORT:
 			/*
 			 * Only supporting single channel requests
@@ -662,6 +679,17 @@ static void rrm_station_watch_cb(enum station_state state, void *userdata)
 	}
 }
 
+static void rrm_station_watch_destroy(void *user_data)
+{
+	struct rrm_state *rrm = user_data;
+
+	l_debug("");
+
+	rrm_cancel_pending(rrm);
+	rrm->watch_id = 0;
+	rrm->station = NULL;
+}
+
 static void rrm_frame_watch_cb(const struct mmpdu_header *mpdu,
 				const void *body, size_t body_len,
 				int rssi, void *user_data)
@@ -684,8 +712,9 @@ static void rrm_frame_watch_cb(const struct mmpdu_header *mpdu,
 			return;
 		}
 
-		station_add_state_watch(rrm->station, rrm_station_watch_cb,
-						rrm, NULL);
+		rrm->watch_id = station_add_state_watch(rrm->station,
+						rrm_station_watch_cb, rrm,
+						rrm_station_watch_destroy);
 	}
 
 	/*
@@ -757,14 +786,20 @@ static void rrm_state_destroy(void *data)
 	l_free(rrm);
 }
 
-static void rrm_new_state(struct netdev *netdev)
+static void rrm_add_frame_watches(struct rrm_state *rrm)
+{
+	static const uint16_t frame_type = 0x00d0;
+	static const uint8_t prefix[] = { 0x05, 0x00 };
+
+	l_debug("");
+
+	frame_watch_add(rrm->wdev_id, 0, frame_type, prefix, sizeof(prefix),
+					rrm_frame_watch_cb, rrm, NULL);
+}
+
+static struct rrm_state *rrm_new_state(struct netdev *netdev)
 {
 	struct rrm_state *rrm;
-	uint16_t frame_type = 0x00d0;
-	uint8_t prefix[] = { 0x05, 0x00 };
-
-	if (netdev_get_iftype(netdev) != NETDEV_IFTYPE_STATION)
-		return;
 
 	rrm = l_new(struct rrm_state, 1);
 
@@ -772,10 +807,9 @@ static void rrm_new_state(struct netdev *netdev)
 	rrm->ifindex = netdev_get_ifindex(netdev);
 	rrm->wdev_id = netdev_get_wdev_id(netdev);
 
-	frame_watch_add(rrm->wdev_id, 0, frame_type, prefix, sizeof(prefix),
-					rrm_frame_watch_cb, rrm, NULL);
-
 	l_queue_push_head(states, rrm);
+
+	return rrm;
 }
 
 static bool match_ifindex(const void *a, const void *b)
@@ -794,8 +828,12 @@ static void rrm_netdev_watch(struct netdev *netdev,
 
 	switch (event) {
 	case NETDEV_WATCH_EVENT_NEW:
-		rrm_new_state(netdev);
-		return;
+		rrm = rrm_new_state(netdev);
+
+		if (netdev_get_iftype(netdev) == NETDEV_IFTYPE_STATION)
+			rrm_add_frame_watches(rrm);
+
+		break;
 	case NETDEV_WATCH_EVENT_DEL:
 		/*
 		 * This event is triggered by the netdev being removed, which
@@ -806,11 +844,22 @@ static void rrm_netdev_watch(struct netdev *netdev,
 		rrm = l_queue_remove_if(states, match_ifindex,
 						L_UINT_TO_PTR(ifindex));
 		if (rrm) {
-			rrm_cancel_pending(rrm);
+			if (rrm->station && rrm->watch_id)
+				station_remove_state_watch(rrm->station,
+								rrm->watch_id);
+
 			l_free(rrm);
 		}
 
-		return;
+		break;
+	case NETDEV_WATCH_EVENT_IFTYPE_CHANGE:
+		rrm = l_queue_find(states, match_ifindex,
+						L_UINT_TO_PTR(ifindex));
+
+		if (rrm && netdev_get_iftype(netdev) == NETDEV_IFTYPE_STATION)
+			rrm_add_frame_watches(rrm);
+
+		break;
 	default:
 		break;
 	}
